#pragma once

#include <i2c.hpp>
#include "vex-encoder-init.hpp"

using namespace stm32f072xb;

// The default address for an encoder that's just been turned on.
constexpr int I2C_ENCODER_DEFAULT_ADDRESS = 0x60;
constexpr int VEX_STARTING_ADDRESS = 0x20;

// Registers for important data
constexpr int I2CENCODER_DEVICE_INFO = 0x20;
constexpr int I2C_ENCODER_ADDRESS_REGISTER = 0x4D;
constexpr int I2C_ENCODER_POSITION_REGISTER = 0x40;
constexpr int I2C_ENCODER_VELOCITY_REGISTER = 0x44;
constexpr int I2C_ENCODER_ZERO_REGISTER = 0x4A;
constexpr int I2C_ENCODER_RESET_ALL_DEVICES = 0x4E;
constexpr int I2C_ENCODER_UNTERMINATE_REGISTER = 0x4B;
constexpr int I2C_ENCODER_TERMINATE_REGISTER = 0x4C;

// Default number of ticks per encoder revolution
constexpr int TICKS = 8;

// Encoder revolutions to Output rotations
// 1 (output rotation) / 39.2 (Encoder revolution)
constexpr float MOTOR_393_TORQUE_ROTATION = static_cast<float>(1 / 39.2);

class VexEncoder {
   public:
	static inline VexEncoder* lastEncoder = nullptr;

	VexEncoder(VexEncoderInit& encoderinit);
	~VexEncoder() = default;
	float getPosition();
	float getMaxPosition();
	uint8_t getAddress();
	void setAddress(uint8_t newAddress);

	void updatePosition();
	void unterminate();
	void terminate();
	void accessRegister(uint8_t reg);
	void zero();

	static void resetAllEncoders(uint8_t numberOfMotor, I2c& i2c);

   private:
	I2c& _i2c;
	uint8_t _address;
	float _position;
	float _maxPosition;

	void monitorVexPosition();
	long getRawPosition();

	static void _accessRegister(uint8_t reg, uint8_t address, I2c& i2c);
	static void _terminate(uint8_t address, I2c& i2c);
	static void _unterminate(uint8_t address, I2c& i2c);
	static void _setAddress(uint8_t newAddress, I2c& i2c);
};
